#pragma once
#ifndef __CONVERSIONS_H__
#define __CONVERSIONS_H__


/**************************************************************************************************
    Description: Get string from string table in resources by resource ID
---------------------------------------------------------------------------------------------------
    Inputs: resource ID
    Output: resource string if success, few question mars otherwise
**************************************************************************************************/
CString ResStr(UINT nIdResourceString)
{
    CString Str;
    if (!Str.LoadString(nIdResourceString))
    {
#ifdef _DEBUG
        Str.Format(TEXT("???_%u_???"), nIdResourceString);
#else
        Str = TEXT("???");
#endif
    }
    return Str;
}

/**************************************************************************************************
    Description: Convert number to string decimal form
---------------------------------------------------------------------------------------------------
    Inputs: nNum - converted number
            nMinWidth - not required - if it is greater than 1 add few leading zeros
                        (string length will be at least nMinWidth)
    Output: String containing number in decimal form
**************************************************************************************************/
CString IntToStr(INT nNum, INT nMinWidth = 0)
{
    CString strNum, strFormat = TEXT("%d");
    if (nMinWidth > 1)
    {
        strFormat.Format(TEXT("%%0%dd"), nMinWidth);
    }
    strNum.Format(strFormat, nNum);
    return strNum;
}

/**************************************************************************************************
    Description: Convert number to string in HEX form
---------------------------------------------------------------------------------------------------
    Inputs: nNum - converted number
            nMinWidth - not required - if it is greater than 1 add few leading zeros
                        (string length will be at least nMinWidth)
    Output: String containing number in HEX form
**************************************************************************************************/
CString IntToStrHex(INT nNum, INT nMinWidth = 0)
{
    CString strNum, strFormat = TEXT("%X");
    if (nMinWidth > 1)
    {
        strFormat.Format(TEXT("%%0%dX"), nMinWidth);
    }
    strNum.Format(strFormat, nNum);
    return strNum;
}

/**************************************************************************************************
    Description: Convert number in HEX form saved in string to number
---------------------------------------------------------------------------------------------------
    Inputs: zero retminated string to be converted
    Output: Converted number if success, zero otherwise
**************************************************************************************************/
INT HexStrToInt(const TCHAR *strNum)
{
    INT nNum, nConverted;
    CString strNumU(strNum);
    strNumU.MakeUpper();
#ifdef UNICODE
    nConverted = swscanf_s(strNumU.GetBuffer(100), TEXT("%X"), &nNum);
#else
    nConverted = sscanf_s(strNumU.GetBuffer(100), TEXT("%X"), &nNum);
#endif
    return (nConverted == 1) ? nNum : 0;
}


/**************************************************************************************************
    Description: Parse input string, remove all non HEX numbers (allowed are 0..9,A..F,a..f),
                 start from the end of the input string, take two chars and convert it to
                 one-byte number... (see examples)
---------------------------------------------------------------------------------------------------
    Inputs: strHex - Input string
            pbyData - Buffer where could be recognized data saved
            nDataSize - Size of buffer pbyData
    Output: Number of recognized bytes
    Examples: string "123456" converted to array {0x12, 0x34, 0x56}
              string  "12345" converted to array {0x01, 0x23, 0x45}
              string "1X34.5" converted to array {0x13, 0x45}
**************************************************************************************************/
INT HexStrToArray(const TCHAR *strHex, BYTE *pbyData, INT nDataSize)
{
    INT nCount = 0;
    CString str(strHex);
    CString strTmp, strNum;
    strTmp.Empty();

    str.MakeUpper(); //Converting chars to upper case makes easier parsing
    if (str.GetLength() <= nDataSize && pbyData != NULL)
    {
        for (INT nIndex = 0; nIndex < nDataSize; nIndex++)
        {
            pbyData[nIndex] = 0;
        }

        //Remove not allowed chars
        for (INT nIndex = 0; nIndex < str.GetLength(); nIndex++)
        {
            if ((str[nIndex] >= 'A' && str[nIndex] <= 'F') ||
                (str[nIndex] >= '0' && str[nIndex] <= '9'))
            {
                strTmp.AppendChar(str[nIndex]);
            }
        }
        nCount = strTmp.GetLength() / 2;
        BOOL bOdd = (strTmp.GetLength() % 2) != 0;
        for (INT nIndex = 0; nIndex < nCount; nIndex++)
        {
            strNum.Empty();
            strNum.AppendChar(strTmp[bOdd + 2*nIndex + 0]);
            strNum.AppendChar(strTmp[bOdd + 2*nIndex + 1]);
            pbyData[bOdd + nIndex] = HexStrToInt(strNum);
        }
        if (bOdd)
        {
            nCount++;
            strNum.Empty();
            strNum.AppendChar(strTmp[0]);
            pbyData[0] = HexStrToInt(strNum);
        }
    }
    return nCount;
}

/**************************************************************************************************
    Description: Convert time from four INT variables to string in format H:MM:SS.mmm
---------------------------------------------------------------------------------------------------
    Inputs: No comment - its clear
    Output: Correct time string if success or '---:--:--.---'
**************************************************************************************************/
CString TimeToStr(INT nHours, INT nMinutes, INT nSeconds, INT nMiliseconds)
{
    CString strTime("---:--:--.---");
    if (nHours >= 0 &&
        nMinutes >= 0 && nMinutes < 60 &&
        nSeconds >= 0 && nSeconds < 60 &&
        nMiliseconds >= 0 && nMiliseconds < 1000)
    {
        strTime.Format(TEXT("%d:%02d:%02d.%03d"), nHours, nMinutes, nSeconds, nMiliseconds);
    }
    return strTime;
}

/**************************************************************************************************
    Description: Convert max. 8 bytes from data array to string in HEX format
                 Bytes are separated by space
---------------------------------------------------------------------------------------------------
    Inputs: pbyData - poiter to data array
            nCount - real number of bytes
    Output: Converted data string if success or '-- -- -- -- -- -- -- --'
**************************************************************************************************/
CString DataToStr(BYTE *pbyData, DWORD dwCount)
{
#define MIN(A, B) ( (A < B) ? A : B )
    CString strRet("-- -- -- -- -- -- -- --");
    if (pbyData != NULL && dwCount >= 0)
    {
        strRet.Empty();
        for (DWORD dwIndex = 0; dwIndex < MIN(dwCount, 8); dwIndex++)
        {
            strRet.AppendFormat(TEXT("%02X"), pbyData[dwIndex]);
            if ((dwIndex + 1) != MIN(dwCount, 8))
            {
                strRet.Append(TEXT(" "));
            }
        }
    }
    return strRet;
#undef MIN
}

#endif // !__CONVERSIONS_H__
