#pragma once
#include "StdAfx.h"
#include "FTD2XX.h"
#include <time.h>

#define RX_BUFFER_SIZE 0x10000 //64kB
#define MAX_MESSAGE_SIZE 0x100 //256B
#define MAX_TAG_LENGTH 20
#define MAX_MASK_LENGTH 20
#define MAX_ID_LENGTH 20
#define MAX_DATA_LENGTH 50
#define MAX_COUNT_ERROR_LENGTH 20
#define MAX_PERIOD_LENGTH 20
#define MAX_PERIOD 65535 //2 bytes value

class CDlgMain : public CDialogImpl<CDlgMain>
{
private:
    HINSTANCE m_hInstance;

    HANDLE m_hThreadTimer;
    DWORD m_dwThreadTimerID;

    HANDLE m_hThreadReceive;
    DWORD m_dwThreadReceiveID;

    BYTE m_byRxDataBuffer[RX_BUFFER_SIZE];
    DWORD m_dwRxDataBufferPos; //first free place, 0 means no data

    BOOL m_bCanPause;

    //Editboxes
    ATLControls::CEdit m_ControlEdit_ArrivalTime;
    ATLControls::CEdit m_ControlEdit_IdReceive;
    ATLControls::CEdit m_ControlEdit_DataReceive;
    ATLControls::CEdit m_ControlEdit_Period;
    ATLControls::CEdit m_ControlEdit_IdTransmit;
    ATLControls::CEdit m_ControlEdit_DataTransmit;
    ATLControls::CEdit m_ControlEdit_Tag;
    ATLControls::CEdit m_ControlEdit_Mask;

    //Buttons
    ATLControls::CButton m_ControlButton_Transmit;
    ATLControls::CButton m_ControlButton_Abort;
    ATLControls::CButton m_ControlButton_Init;
    ATLControls::CButton m_ControlButton_Pause;
    ATLControls::CButton m_ControlButton_Stop;
    ATLControls::CButton m_ControlButton_Clear;

    //Radiobuttons
    ATLControls::CButton m_ControlRadio_Type20A;
    ATLControls::CButton m_ControlRadio_Type20B;
    ATLControls::CButton m_ControlRadio_High;
    ATLControls::CButton m_ControlRadio_Low;
    ATLControls::CButton m_ControlRadio_OneWire;
    enum enumOneWireType {
        ONE_WIRE_TYPE_NONE = 0,
        ONE_WIRE_TYPE_NORMAL = 1,
        ONE_WIRE_TYPE_HIGH,
        ONE_WIRE_TYPE_WAKE,
        ONE_WIRE_TYPE_FIRST = ONE_WIRE_TYPE_NORMAL,
        ONE_WIRE_TYPE_LAST = ONE_WIRE_TYPE_WAKE
    };
    enumOneWireType m_eOneWireType;

    //Checkboxes
    ATLControls::CButton m_ControlCheck_Periodic;
    ATLControls::CButton m_ControlCheck_Remote;
    ATLControls::CButton m_ControlCheck_ShowTransmited;
    ATLControls::CButton m_ControlCheck_ListeningMode;

    //Labels
    ATLControls::CStatic m_ControlLabel_Errors;
    ATLControls::CStatic m_ControlLabel_Count;

    //Comboboxes
    ATLControls::CComboBox m_ControlCombo_BitRate;

    INT GetDriverSpeed();
    VOID SetOneWireType(enumOneWireType nType);
    BOOL SendMessageToMC(BYTE byMsgType, BYTE *pbyData, BYTE byDataLength);
    VOID DisableTransmit(BYTE byMCx);
    VOID DisableTransmitToAll();
    VOID CodeID(INT nID, BOOL bRemote, BYTE *pbyIDT1, BYTE *pbyIDT2, BYTE *pbyIDT3, BYTE *pbyIDT4);
    DWORD DecodeID(BYTE byIDT1, BYTE byIDT2, BYTE byIDT3, BYTE byIDT4, BYTE byCONCH, BOOL *pbIDE, BOOL *pbRemote);
    BYTE CRC(BYTE *pbyMessage, DWORD dwLength);
    VOID DisplayMessage(BYTE *pbyMessage, DWORD dwLength);

public:
    CDlgMain(HINSTANCE hInstance);
    ~CDlgMain();

    enum { IDD = IDD_DLG_MAIN };

BEGIN_MSG_MAP(CDlgMain)
    MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
    MESSAGE_HANDLER(WM_SHOWWINDOW, OnShowWindow)
    MESSAGE_HANDLER(WM_CLOSE, OnClose)

    COMMAND_HANDLER(IDC_BUTTON_TRANSMIT, BN_CLICKED, OnButtonTransmitClicked)
    COMMAND_HANDLER(IDC_BUTTON_ABORT, BN_CLICKED, OnButtonAbortClicked)
    COMMAND_HANDLER(IDC_BUTTON_INIT, BN_CLICKED, OnButtonInitClicked)
    COMMAND_HANDLER(IDC_BUTTON_PAUSE, BN_CLICKED, OnButtonPauseClicked)
    COMMAND_HANDLER(IDC_BUTTON_STOP, BN_CLICKED, OnButtonStopClicked)
    COMMAND_HANDLER(IDC_BUTTON_CLEAR, BN_CLICKED, OnButtonClearClicked)
    COMMAND_HANDLER(IDC_RADIO_HIGH, BN_CLICKED, OnRadioHighSpeedClicked)
    COMMAND_HANDLER(IDC_RADIO_LOW, BN_CLICKED, OnButtonLowSpeedClicked)
    COMMAND_HANDLER(IDC_RADIO_ONE_WIRE, BN_CLICKED, OnButtonOneWireClicked)
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

    LRESULT OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    LRESULT OnShowWindow(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    LRESULT OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);

    LRESULT OnButtonTransmitClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnButtonAbortClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnButtonInitClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnButtonPauseClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnButtonStopClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnButtonClearClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnRadioHighSpeedClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnButtonLowSpeedClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    LRESULT OnButtonOneWireClicked(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);


    BOOL m_bThreadTimerTerminate; //TRUE if timer thread should be terminated
    BOOL m_bThreadTimerPaused; //TRUE if device should be paused

    BOOL m_bThreadReceiveTerminate; //TRUE if receiving thread should be terminated
    BOOL m_bThreadReceivePaused; //TRUE if receiving should be paused

    FT_HANDLE m_FT_Handle; //Handle to FT library
    BOOL m_bDevicePresent; //TRUE if device is connected

    //This function is called from notification thread
    //Calling is not treated for exclusive access so
    //DO NOT call this method in more than one thread!!!
    VOID OnDataReceived(BYTE *pbyData, DWORD dwCount);
};
